
/***********************************************                                                 
*   Firmware version:2.0
*   autore: 
*   Data:
*   MCU: PIC16F876A                            
*   Piattaforma hardware: Micro-GT mini  
*   Piattaforma software: MPLAB X
*   Compilatore: HI-TECH C Compiler for PIC10/12/16 MCUs(Lite Mode) V9.70   
***********************************************/

//----------------- Define per l'utilizzo del diplay ---------------
#define	LCD_D4		RB1 //bus dati
#define	LCD_D5		RB2 //bus dati
#define	LCD_D6		RB3 //bus dati
#define	LCD_D7		RB4 //bus dati

#define	LCD_E		RC0 //ENABLED
#define	LCD_RS		RC1 //RS
//-------------------------------------------------------------------

/*LCD interface header file
  write a byte to the LCD in 4 bit mode */

extern void lcd_write(unsigned char);

/* Clear and home the LCD */

extern void lcd_clear(void);

/* write a string of characters to the LCD */

extern void lcd_puts(const char * s);

/* Go to the specified position */

extern void lcd_goto(unsigned char pos);
	
/* intialize the LCD - call before anything else */

extern void lcd_init(void);

extern void lcd_putch(char);

/*	Set the cursor position */

#define	lcd_cursor(x)	lcd_write(x|0x80)
//------------------------------------------------------------------------------------//


//*************************************************************************************
// Funzione pausa per attendere che il Display stabilizzi i comandi ricevuti e abbia
//il tempo di eseguirli.
//il parametro da passare e' 65536 max, il valore tipico  maggiore di 650ms.

void pause(unsigned char temps)
	{
	temps--;
	for(;temps!=0;temps--)
		{
#if  defined(_16F1937) || defined(_16LF1937) || defined(_16LF1824) || defined(_16F1824)|| defined(_16LF1828)|| defined(_16LF1828)
#else
		DelayMs(1);
#endif
		DelayMs(1);
		}
	}
//-----------------------------
//Funzione che genera una pausa di 1 ms
//-----------------------------
void pause_ms(unsigned int temps)
	{
	unsigned char a;	
	
	for(; temps > 0 ; temps--)
		{
		for(a=195;a!=0;a--)
			{
			asm("clrwdt");
			asm("nop");
			}
		if(temps!=1)
			{
			for(a=3;a!=0;a--);
			}
		}
	}

//---------------------------------------------------------
//lcd_write : Scrive un byte sul display LCD nella modalita' a 4 bit
//---------------------------------------------------------
void lcd_write(unsigned char c)
	{
	pause(4);
	LCD_D7 = 0;
	LCD_D6 = 0;
	LCD_D5 = 0;
	LCD_D4 = 0;
	if((c & 0b10000000) != 0)
		LCD_D7 = 1;
	if((c & 0b01000000) != 0)
		LCD_D6 = 1;
	if((c & 0b00100000) != 0)
		LCD_D5 = 1;
	if((c & 0b00010000) != 0)
		LCD_D4 = 1;
		
		
	LCD_E=1;
	DelayMs(1);
	LCD_E=0;

	LCD_D7 = 0;
	LCD_D6 = 0;
	LCD_D5 = 0;
	LCD_D4 = 0;
	if((c & 0b00001000) != 0)
		LCD_D7 = 1;
	if((c & 0b00000100) != 0)
		LCD_D6 = 1;
	if((c & 0b00000010) != 0)
		LCD_D5 = 1;
	if((c & 0b00000001) != 0)
		LCD_D4 = 1;

	LCD_E=1;
	DelayMs(1);
	LCD_E=0;
	pause(4);
	}


//------------------------------------------
// lcd_clear : Cancella il Display e posizione il cursore nella prima cella
//------------------------------------------
void lcd_clear(void)
	{
	LCD_RS = 0;
	lcd_write(0x1);
	pause_ms(40);
	}

//-----------------------------------------------------
// lcd_puts : Scrive una stringa di caratteri nel display LCD dalla posizione attuale
//-----------------------------------------------------
void lcd_puts(const char * s)
	{
	LCD_RS = 1;	// scrive un carattere
	while(*s)
		lcd_write(*s++);
	}

//--------------------------------------------------
//--- lcd_putch : write one character to the LCD ---
//--------------------------------------------------
void lcd_putch(char c)
	{
	LCD_RS = 1;	// write characters
	lcd_write( c );
	}

//-----------------------------------------------
//--- lcd_goto : Posizione il cursore nel punto richiesto
//-----------------------------------------------
void lcd_goto(unsigned char pos)

	{
        //char t;
	LCD_RS = 0;
        if (pos<16)
	   lcd_write(0x80|pos);
        else
        {
            //t = pos-16;
            lcd_write(0b11000000|pos-16);
            }
	}
	
//-----------------------------------------------------------
//--- lcd_init : Inizializza il display nella modalita' a 4 bit
//-----------------------------------------------------------
void lcd_init(void)
	{
	LCD_RS = 0;
	LCD_E = 0;
//	LCD_RW = 0;
	
	pause_ms(30);	// aspetta 15milli secondi dopo l'alimentazione del display
	LCD_D7 = 0;
	LCD_D6 = 0;
	LCD_D5 = 1;
	LCD_D4 = 1;

	LCD_E=1;
	DelayMs(1);
	LCD_E=0;
	pause_ms(30);
	LCD_E=1;
        DelayMs(1);
	LCD_E=0;
	pause(150);
	LCD_E=1;
        DelayMs(1);
	LCD_E=0;
	pause(150);
	
	LCD_D7 = 0;		// seleziona modalita' a 4 bit
	LCD_D6 = 0;
	LCD_D5 = 1;
	LCD_D4 = 0;

	LCD_E=1;
        DelayMs(1);
	LCD_E=0;

	lcd_write(0x28); // Imposta la lunghezza dell'interfaccia
	lcd_write(0b00001100); // Accende display, accende cursore,lampeggia cursore
	lcd_clear();	// pulisce lo schermo
	lcd_write(0x6); // imposta la modalita' di scrittura

	}

/* ---------- Esempio inizializzazione display con  Presentazione iniziale --------
void main(){
    TRISD=0;
    TRISE=0;
    lcd_init();
    lcd_puts("G-Tronic Robotic");
    DelayS(3);
    lcd_clear();
    lcd_puts("Micro-GT mini   ");
while(1){
    CLRWDT();
}
}
*/
